/* spmfilter - diskarchive-plugin for spmfilter mail filtering framework
 * Copyright (C) 2009-2010 Werner Detter and SpaceNet AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdlib.h>
#include <string.h>
#include <spmfilter.h>
#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#define THIS_MODULE "diskarchive"

/* settings */
static char *path = NULL;
static int by_user = 0;

/** saves message to archive directory
 *
 * \param in_path
 *
 * \returns 0 if message was successfully archived
 */
static int do_archive(char *in_path) {
	char *dst;
	char *filename;

	/* generate unique filename */
	filename = smf_core_get_maildir_filename();

	/* generate path and save */
	dst = g_strdup_printf("%s/%s",in_path, filename);
	smf_session_to_file(dst);
	TRACE(TRACE_INFO, "created copy in local disk-archive: %s", dst);
	g_free(dst);

	return 0;
}

/** create directory for archiving
 *
 * \param in_path (absolute path of archive directory)
 *
 * \returns 0 if directory was successfully created, 1 if creation failed
 */
static int create_dir(char *in_path) {
	struct stat st;

	if(stat(in_path,&st) == 0) {
		TRACE(TRACE_DEBUG, "dir exists: %s", in_path);
		return 0;
	} else {
		if(mkdir(in_path, 0755) == -1) {
			TRACE(TRACE_DEBUG, "failed to create directory: %s", in_path);
			return -1;
		} else {
			TRACE(TRACE_DEBUG, "dir created: %s", in_path);
			return 0;
		}
	}
}



/* START */
int load(SMFSession_T *session) {
	int i;
	char *dest;
	SMFSettings_T *settings = smf_settings_get();

	/* check for settings */
	if(smf_settings_group_load(THIS_MODULE) !=0 ) {
		TRACE(TRACE_ERR, "no configuration found in spmfilter.conf");
		return -1;
	}

	/* grab settings from spmfilter.conf */
	path = smf_settings_group_get_string("path");
	by_user = smf_settings_group_get_boolean("by_user");
	smf_settings_group_free();

	/* debug */
	TRACE(TRACE_DEBUG, "path: %s", path);
	TRACE(TRACE_DEBUG, "by_user: %d", by_user);
	
	/* if no lookup backend is defined, but by_user is set to true, */
	/* we manually set by_user to false and generate a warning */
	if(NULL == settings->backend) {
		TRACE(TRACE_WARNING, "warning: no backend for \
			user lookups defined, manually set \"by_user\" to \"false\"");
		by_user = 0;
	}

	/* check for archive directory specified in config, create if missing */
	if(create_dir(path) == -1) {
		TRACE(TRACE_ERR, "unable to create archive-dir: %s", path);
		return -1;
	}

	/* simple procedure of archiving if no distinction is needed */
	if(by_user == 0) {
		if(do_archive(path) == -1) {
			TRACE(TRACE_ERR,"unable to archive message");
			return -1;
		}
	} else if (by_user == 1) {
		/* smtp engine sender */
		if (session->envelope_from != NULL) {
			/* check if sender is local for smtp engine */
			if(session->envelope_from->is_local == 1) {
				dest = g_strdup_printf("%s/%s", path, session->envelope_from->addr);
				/* create archive directory if nonexistent */
				if(create_dir(dest) == -1) {
					TRACE(TRACE_ERR, "unable to create archive-dir: %s", dest);
					if(dest != NULL)
						g_free(dest);
					return -1;
				}
				/* archive message */
				if (do_archive(dest) == -1) {
					TRACE(TRACE_ERR,"unable to archive message");
					if(dest != NULL)
						g_free(dest);
					return -1;
				}
				if(dest != NULL)
					g_free(dest);
			}
		/* pipe engine sender */
		} else if (session->message_from != NULL) {
			/* check if sender is local for pipe engine */
			if(session->message_from->is_local == 1) {
				dest = g_strdup_printf("%s/%s", path, session->message_from->addr);
				/* create archive directory if nonexistent */
				if(create_dir(dest) == -1) {
					TRACE(TRACE_ERR, "unable to create archive-dir: %s", dest);
					if(dest != NULL)
						g_free(dest);
					return -1;
				}
				/* archive message */
				if (do_archive(dest) == -1) {
					TRACE(TRACE_ERR,"unable to archive message");
					if(dest != NULL)
						g_free(dest);
					return -1;
				}
				if(dest != NULL)
					g_free(dest);
			}
		}
		/* smtp engine recipient */
		if(session->envelope_to != NULL) {
			for(i=0; i < session->envelope_to_num; i++) {
				/* check if rcpt is local */
				if(session->envelope_to[i]->is_local == 1) {
					/* if local sender != local rcpt */
					if(strcmp(session->envelope_to[i]->addr, session->envelope_from->addr) != 0) {
						dest = g_strdup_printf("%s/%s",path,session->envelope_to[i]->addr);
						/* create archive directory if nonexistent */
						if(create_dir(dest) == -1) {
							TRACE(TRACE_ERR, "unable to create archive-dir: %s", dest);
							if(dest != NULL)
								g_free(dest);
							return -1;
						}
						/* archive message */
						if (do_archive(dest) == -1) {
							TRACE(TRACE_ERR,"unable to archive message");
							if(dest != NULL)
								g_free(dest);
							return -1;
						}
						if(dest != NULL)
							g_free(dest);
					}
				}
			}
		/* pipe engine recipient */
		} else if (session->message_to != NULL) {
			for(i=0; i < session->message_to_num; i++) {
				if(session->message_to[i]->is_local == 1) {
					if(strcmp(session->message_to[i]->addr, session->message_from->addr) != 0) {
						dest = g_strdup_printf("%s/%s",path,session->message_to[i]->addr);
						/* create archive directory if nonexistent */
						if(create_dir(dest) == -1) {
							TRACE(TRACE_ERR, "unable to create archive-dir: %s", dest);
							if(dest != NULL)
								g_free(dest);
							return -1;
						}
						/* archive message */
						if (do_archive(dest) == -1) {
							TRACE(TRACE_ERR,"unable to archive message");
							if(dest != NULL)
								g_free(dest);
							return -1;
						}
						if(dest != NULL)
							g_free(dest);
					}
				}
			}
		}
	} else {
		/* wrong value defined for config-option by_user */
		TRACE(TRACE_ERR,"wrong value for config option by_user");
		return -1;
	}
return 0;
}
